package entity;

import java.util.List;

public interface PizzaInterface {

	/** Fuegt die uebergebene Zutat zur Pizza hinzu
	 * und gibt diese zurueck.
	 * @param z hinzuzufuegende Zutat
	 * @return this mit Zutat z 
	 */
	public PizzaInterface zutatHinzu(Zutat z);

	/** Entfernt eine Zutat mit deem Namen name von der
	 * Pizza und gibt diese wieder zurueck. Existiert keine
	 * Zutat mit dem Namen, wird die Pizza nicht veraendert
	 * @param name Name der zu entfernenden Zutat
	 * @return this ohne Zutat mit Namen namen
	 */
	public PizzaInterface zutatWeg(String name);

	/** Gibt die minimale Backzeit der Pizza in Sekunden,
	 * die sich durch bestimmte Zutaten verlaengern kann.
	 * @return Backzeit entspricht Mindestwartezeit 
	 */
	public int backzeit();
	
	/** Berechnet den  Gesamtpreis der Pizza unter
	 * Beruecksichtigung der aktuellen Zutaten.
	 * @return Gesamtpreis
	 */
	public int preis();

	/** Gibt die konkrete Rechnung zur Pizza bestehend aus
	 * der Pizza und den Zutaten mit dem Gesamtpreis aus
	 * @return vollstaendige Detailrechnung
	 */
	public String rechnung();

	/** Gibt den Typen der Pizza, entspricht dem Namen
	 * auf der Speisekarte.
	 * @return Typ
	 */
	public String getTyp();
	
	/** Gibt die Liste der Zutaten dieser Pizza.
	 * 
	 * @return alle Zutaten
	 */
	public List<Zutat> getZutaten();

	/** Berechnet eine Kopie des Objektes, es ist zu beachten,
	 * dass der Rueckgabetyp bei der Realisierung auch einer
	 * sein kann, der das Interface realisiert.
	 * 
	 * @return Objektkopie
	 */
	public PizzaInterface clone();
}