package main;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import com.github.stefanbirkner.systemlambda.SystemLambda;

import control.Pizzeria;
import entity.Pizza;
import entity.PizzaInterface;
import entity.Zutat;

class SystemTest {
   
  private Pizzeria pi;
  private PizzaInterface pizza; // schmuddelig, so in lambda-Ausdruck nutzbar
  
  @BeforeEach
  public void setUp() {
    this.pi = new Pizzeria();
  }
  
  private void pizzaAuswahl(int nr) throws Exception{
    String[] inputs = {"" + nr};
    SystemLambda.withTextFromSystemIn(inputs)
    .execute(() -> {
      pizza = this.pi.pizzaAuswaehlen();
    });
  }
  
  private void gesamtPizzaAuswahl(int... nr) throws Exception{
    String[] inputs = new String[nr.length];
    for(int i = 0 ; i < nr.length; i++) {
      inputs[i] = "" + nr[i];
    }
    SystemLambda.withTextFromSystemIn(inputs)
    .execute(() -> {
      pizza = Main.pizzaBestellen(this.pi);
    });
  }

  @Test
  public void testStrukturKlassePizza1() throws Exception{
    Class<?> clazz = Pizza.class;
    for (Field v: clazz.getDeclaredFields()) { 
      if (v.getType() == List.class || v.getType() == ArrayList.class) {
        throw new IllegalStateException("Pizza darf keine Liste mehr enthalten.");
      }
    }
  }
  
  @Test
  public void testStrukturKlassePizza2() throws Exception{
    Class<?> clazz = Pizza.class;
    for (Field v: clazz.getDeclaredFields()) { 
      if (v.getType() == PizzaInterface.class) {
        throw new IllegalStateException("Pizza soll dekoriert werden und "
            + "dekoriert nicht selber, keine Objektvariable vom Typ "
            + "PizzaInterface.");
      }
    }
  }
  
  @Test
  public void testStrukturKlasseZutat() throws Exception{
    Class<?> clazz = Zutat.class;
    boolean gefunden = false;
    for (Field v: clazz.getDeclaredFields()) { 
      if (v.getType() == PizzaInterface.class) {
        gefunden = true;
      }
    }
    Assertions.assertTrue(gefunden, "Zutat sollte als Dekorierer eine "
        + "Objektvariable vom Typ PizzaInterface haben");
  }
  
  
  @Test
  public void testPizza1() throws Exception{
    this.pizzaAuswahl(1);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"), ergebnis);
    Assertions.assertFalse(ergebnis.contains("Champignon"), ergebnis);
    Assertions.assertFalse(ergebnis.contains("Prosciutto"), ergebnis);
    Assertions.assertEquals(600, pizza.preis(), ergebnis);
    Assertions.assertEquals(380, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza2() throws Exception{
    this.pizzaAuswahl(2);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"), ergebnis);
    Assertions.assertTrue(ergebnis.contains("Champignon"), ergebnis);
    Assertions.assertFalse(ergebnis.contains("Prosciutto"), ergebnis);
    Assertions.assertEquals(750, pizza.preis(), ergebnis);
    Assertions.assertEquals(380, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza3() throws Exception{
    this.pizzaAuswahl(3);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"), ergebnis);
    Assertions.assertTrue(ergebnis.contains("Champignon"), ergebnis);
    Assertions.assertTrue(ergebnis.contains("Prosciutto"), ergebnis);
    Assertions.assertEquals(950, pizza.preis(), ergebnis);
    Assertions.assertEquals(440, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza4() throws Exception{
    this.gesamtPizzaAuswahl(1,9,5,0,0);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"), ergebnis);
    Assertions.assertTrue(ergebnis.contains("Champignon"), ergebnis);
    Assertions.assertTrue(ergebnis.contains("Prosciutto"), ergebnis);
    Assertions.assertEquals(950, pizza.preis(), ergebnis);
    Assertions.assertEquals(440, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza5() throws Exception{
    this.gesamtPizzaAuswahl(3,0,2,1,0);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"));
    Assertions.assertFalse(ergebnis.contains("Champignon"));
    Assertions.assertFalse(ergebnis.contains("Prosciutto"));
    Assertions.assertEquals(600, pizza.preis(), ergebnis);
    Assertions.assertEquals(380, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza6() throws Exception{
    this.gesamtPizzaAuswahl(3,0,1,1,0);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"));
    Assertions.assertFalse(ergebnis.contains("Champignon"));
    Assertions.assertFalse(ergebnis.contains("Prosciutto"));
    Assertions.assertEquals(600, pizza.preis(), ergebnis);
    Assertions.assertEquals(380, pizza.backzeit(), ergebnis);
  }
  
  @Test
  public void testPizza7() throws Exception{
    this.gesamtPizzaAuswahl(1,3,3,0,0);
    String ergebnis = pizza.rechnung();
    Assertions.assertTrue(ergebnis.contains("Pizza"));
    Assertions.assertFalse(ergebnis.contains("Champignon"));
    Assertions.assertTrue(Pattern.matches("(?s).*Ei.*Ei.*"
        , ergebnis));
    Assertions.assertEquals(960, pizza.preis(), ergebnis);
    Assertions.assertEquals(380, pizza.backzeit(), ergebnis);
  }

}
